package mastermind.javafx;

import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.geometry.Insets;
import javafx.scene.control.Button;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Border;
import javafx.scene.layout.BorderStroke;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.text.Text;
import mastermind.Mastermind;

import java.net.URL;
import java.util.ResourceBundle;

public class Controller implements Initializable {

    @FXML private FlowPane pinPanel;
    @FXML private FlowPane currentGuessPanel;
    @FXML private VBox previousGuessPanel;
    @FXML private Button submitButton;
    @FXML private Button clearButton;

    //Private mastermind variables
    private Mastermind mastermind;
    private int maxTurns;
    private int pinNumber;
    private Color[] pinColors;
    private char[] pinColorLetters;
    private String currentGuessString;

    public Controller() {
        pinColorLetters = new char[]{'R', 'G', 'B', 'O', 'Y', 'P'};
        pinColors = new Color[]{Color.RED, Color.GREEN, Color.BLUE, Color.ORANGE, Color.YELLOW, Color.PURPLE};
        maxTurns = 10;
        pinNumber = 4;
        currentGuessString = "";
        mastermind = new Mastermind(maxTurns, pinNumber, pinColorLetters);
    }

    @Override
    public void initialize(URL location, ResourceBundle resources) {

        //Add some circles to the pin panel
        for (int i = 0; i < pinColors.length; i++) {
            Circle circle = new Circle();
            circle.setRadius(20);
            circle.setFill(pinColors[i]);
            circle.setOnMouseClicked(new PinMouseClickListener(pinColorLetters[i]));
            pinPanel.getChildren().add(circle);
        }

        //Set the size of the buttons to full width of parent
        //Add onclick listeners for the 2 buttons
        submitButton.setMaxWidth(Double.MAX_VALUE);
        clearButton.setMaxWidth(Double.MAX_VALUE);
        submitButton.setOnAction(event -> {
            if (mastermind.getCurrentTurn() < maxTurns && !mastermind.isWon() && currentGuessString.length() == pinNumber) {

                //Submit current guess string to mastermind
                mastermind.takeTurn(currentGuessString.toCharArray());
                addPreviousGuess(currentGuessString, mastermind.getCorrectlyColored(), mastermind.getCorrectlyPlaced());
                resetGuess();
                winConditionCheck();
            }
            else
                winConditionCheck();
        });
        clearButton.setOnAction(event -> {
            resetGuess();
        });
    }

    //This will update the current guess panel so that it shows the player what they
    //Currently have as a guess
    public void updateGuessPanel() {

        char[] guessLetters = currentGuessString.toCharArray();
        Circle[] pins = new Circle[guessLetters.length];
        for (int i = 0; i < guessLetters.length; i++) {
            for (int j = 0; j < pinColorLetters.length; j++) {
                if (guessLetters[i] == pinColorLetters[j]) {

                    //Create the pin
                    pins[i] = new Circle();
                    pins[i].setRadius(10);
                    pins[i].setFill(pinColors[j]);
                    break;
                }
            }
        }
        currentGuessPanel.getChildren().clear();
        currentGuessPanel.getChildren().addAll(pins);
    }

    //This will set the current guess string to empty and
    //update the current guess display so that there is nothing as
    //a current guess
    public void resetGuess() {

        currentGuessString = "";
        updateGuessPanel();
    }

    //This function will add a submitted guess to the previous
    //guess panel so that the player knows what they have already
    //guessed previously. Some text and pins are add to the screen
    public void addPreviousGuess(String guessString, int correctColor, int correctPlaced) {

        Text correctString = new Text("C: " + correctColor + " P: " + correctPlaced);
        char[] guessLetters = guessString.toCharArray();
        Circle[] pins = new Circle[guessLetters.length];
        for (int i = 0; i < guessLetters.length; i++) {
            for (int j = 0; j < pinColorLetters.length; j++) {
                if (guessLetters[i] == pinColorLetters[j]) {

                    //Create the pin
                    pins[i] = new Circle();
                    pins[i].setRadius(15);
                    pins[i].setFill(pinColors[j]);
                    break;
                }
            }
        }
        FlowPane flow = new FlowPane();
        flow.setHgap(5);
        flow.setVgap(5);
        flow.getChildren().addAll(pins);
        flow.getChildren().add(correctString);
        previousGuessPanel.getChildren().add(flow);
    }

    //This function will print a message onto the previousGuessPanel
    //So that the user can see the textual interactions with the game
    public void displayMessage(String message) {

        Text text = new Text(message);
        previousGuessPanel.getChildren().add(text);
    }

    //This function will check whether the player has either won or lost
    //The function will also print out the number of turns that the player
    //has left to win the game
    public void winConditionCheck() {
        if (mastermind.getCurrentTurn() < maxTurns) {
            if (mastermind.isWon())
                displayMessage("Congratulations! you have won!");
            else
                displayMessage(maxTurns - mastermind.getCurrentTurn() + " turn(s) left!");
        }
        else {
            if (mastermind.isWon())
                displayMessage("Congratulations! you have won!");
            else
                displayMessage("Unfortunately you have lost the game");
        }
    }

    //This class will be added to the guess pins so that when they are
    //clicked the pin will update the guess string and will update the
    //current guess panel so that the player can see their current
    //guess
    class PinMouseClickListener implements EventHandler<MouseEvent> {

        private char letter;

        PinMouseClickListener(char letter) {
            this.letter = letter;
        }

        @Override
        public void handle(MouseEvent event) {

            if (currentGuessString.length() < pinNumber) {
                currentGuessString += letter;
                updateGuessPanel();
            }
        }
    }
}
